<?php
/**
 * Déclaration de la classe CommandLine\Argument\Option\OptionAbstract.
 */

namespace jrosset\CommandLine\Argument\Option;

use InvalidArgumentException;
use jrosset\CommandLine\Argument\ArgumentAbstract;

/**
 * Classe abstraite pour les arguments de type "option" : --xxx / -x
 *
 * @package CommandLine\Argument\Option
 */
abstract class OptionAbstract extends ArgumentAbstract implements IArgumentOption {
    /**
     * @var string|null L'argument court (cas d'un seul "-"). Null si aucun.
     */
    protected ?string $tagShort;
    /**
     * @var string L'argument long (cas d'un "--").
     */
    protected string $tagLong;

    /**
     * @var boolean Est-ce que l'argument est autorisé plusieurs fois ?
     */
    protected bool $multiple = false;
    /**
     * @var boolean Est-ce que l'argument met fin au parsage ?
     */
    protected bool $isStoppingParse = false;

    /**
     * Crée un nouvel argument de type option.
     *
     * @param string      $name        Le nom.
     * @param string|null $description La description.
     * @param string|null $tagLong     Le tag long.
     * @param string|null $tagShort    Le tag court.
     */
    public function __construct (string $name, ?string $description, ?string $tagLong = null, ?string $tagShort = null) {
        parent::__construct($name, $description);

        $this->setTagLong($tagLong);
        $this->setTagShort($tagShort);
    }

    /**
     * Est-ce la valeur correspond au tag court ou long ?
     *
     * Utilisé par les classes enfants pour savoir si le tag correspond.
     *
     * @param string $arg La valeur à examiner.
     *
     * @return boolean True si le tag correspond, sinon False.
     */
    protected function _parseTag (string $arg): bool {
        if ($this->hasTagShort()) {
            if ($arg == '-' . $this->getTagShort()) {
                return true;
            }
        }

        return $arg == '--' . $this->getTagLong();
    }

    /**
     * Est-ce que la liste de valeur contient un argument de type "option" ?
     *
     * @param string[] $args La liste de valeurs à tester
     *
     * @return string|false La valeur qui une option au False si aucune correspondance
     */
    public static function containsOption (array $args) {
        foreach ($args as $arg) {
            if (preg_match('@^--?[a-zA-Z0-9_-]+@', $arg)) {
                return $arg;
            }
        }

        return false;
    }

    public function getTagShort (): ?string {
        return $this->tagShort;
    }
    /**
     * Définit le tag court.
     *
     * @param string|null $tagShort Le tage court.
     *
     * @return $this
     *
     * @see $tagShort
     */
    public function setTagShort (string $tagShort = null): self {
        $this->tagShort = $tagShort;
        return $this;
    }
    /**
     * Est-ce que l'argument existe en forme courte ?
     *
     * @return boolean True si existe en forme courte, sinon False.
     */
    public function hasTagShort (): bool {
        return !is_null($this->tagShort);
    }

    public function getTagLong (): string {
        return $this->tagLong;
    }
    /**
     * Définit le tag long.
     *
     * Si non fourni, est déduit du {@see $name nom de l'argument} :
     *  - Les "_" sont remplacés par "-"
     *  - Les lettres majuscules sont remplacées par "-" suivit de la lettre en minuscule
     *
     * @param string|null $tagLong Le tag long.
     *
     * @return $this
     *
     * @see $tagLong
     */
    public function setTagLong (string $tagLong = null): self {
        if (is_null($tagLong)) {
            $this->tagLong = preg_replace_callback(
                '/[A-Z_]/',
                function ($matches) {
                    if ($matches[0] == '_') {
                        return '-';
                    }
                    else {
                        return '-' . strtolower($matches[0]);
                    }
                },
                $this->getName()
            );
        }
        else {
            $this->tagLong = $tagLong;
        }

        return $this;
    }

    public function allowMultiple (): bool {
        return $this->multiple;
    }
    /**
     * Définit si l'argument est autorisé plusieurs fois ou non.
     *
     * @param boolean $multiple Argument autorisé plusieurs fois ?
     *
     * @return $this
     * @see $_optional
     */
    public function setMultiple (bool $multiple): self {
        if (!is_bool($multiple)) {
            throw new InvalidArgumentException('La multiplicité n\'est pas un booléen');
        }

        $this->multiple = $multiple;
        return $this;
    }

    public function isStoppingParse (): bool {
        return $this->isStoppingParse;
    }
    /**
     * Définit si l'argument met fin au parsage ou non.
     *
     * @param boolean $stoppingParse Met fin au parsage ?
     *
     * @return $this
     * @see $_optional
     */
    public function setStoppingParse (bool $stoppingParse): self {
        if (!is_bool($stoppingParse)) {
            throw new InvalidArgumentException('La stoppabilité n\'est pas un booléen');
        }

        $this->isStoppingParse = $stoppingParse;
        return $this;
    }
}